import java.awt.*;

public class DrawingWarship {
    protected EntityWarship Warship;
    public EntityWarship GetWarship(){
        return Warship;
    }
    protected IDrawingObjectBlock Blocks;
    private BlockCount _block;
    protected int _startPosX;
    protected int _startPosY;
    private Integer _pictureWidth = null;
    private Integer _pictureHeight = null;
    private int _warshipWidth = 120;
    private int _warshipHeight = 40;

    public DrawingWarship(int speed, float weight, Color bodyColor, int blockForm)
    {
        Warship = new EntityWarship(speed, weight, bodyColor);
        Blocks= GetFormOfBlock(blockForm);
        Blocks.SetBlockCount(2*(int)(Math.random()*3+1));
    }
    public DrawingWarship(int speed, float weight, Color bodyColor, IDrawingObjectBlock blockForm)
    {
        Warship = new EntityWarship(speed, weight, bodyColor);
        Blocks= blockForm;
    }
    public DrawingWarship(EntityWarship warship,IDrawingObjectBlock block){
        Warship = warship;
        Blocks = block;
    }

    public IDrawingObjectBlock GetFormOfBlock(int FormOfBlock){
        BlockForm temp = null;
        for (BlockForm form:BlockForm.values()) {
            if(form.Value==FormOfBlock){
                temp = form;
                break;
            }
        }
        if(temp==null)
            return null;
        switch (temp){
            case Rect:
                return new DrawingBlock(_block);
            case Round:
                return new DrawingRoundBlocks(_block);
            case Triangle:
                return new DrawingTriangleBlocks(_block);
        }
        return null;
    }

    protected DrawingWarship(int speed, float weight, Color bodyColor,int warshipWidth, int warshipHeight)
    {
        Warship = new EntityWarship(speed, weight, bodyColor);
        Blocks= new DrawingBlock(_block);
        Blocks.SetBlockCount(2*(int)(Math.random()*3+1));
        _warshipWidth=warshipWidth;
        _pictureHeight=warshipHeight;
    }

    public void SetPosition(int x, int y, int width, int height)
    {
        if (x >= 0 && x+_warshipWidth <= width && y >= 0 && y+_warshipHeight <= height)
        {
            _startPosX = x;
            _startPosY = y;
            _pictureWidth = width;
            _pictureHeight = height;
        }
        else return;
    }

    public void MoveTransport(Direction direction)
    {
        if (_pictureWidth == null || _pictureHeight == null)
        {
            return;
        }
        switch (direction)
        {
            case Right:
                if (_startPosX + _warshipWidth + Warship.Step < _pictureWidth)
                {
                    _startPosX+=Warship.Step;
                }
                break;

            case Left:
                if (_startPosX- Warship.Step >= 0)
                {
                    _startPosX -= Warship.Step;
                }
                break;

            case Up:
                if (_startPosY - Warship.Step >= 0)
                {
                    _startPosY -= Warship.Step;
                }
                break;

            case Down:
                if (_startPosY + _warshipHeight + Warship.Step < _pictureHeight)
                {
                    _startPosY += Warship.Step;
                }
                break;
        }
    }

    public void DrawTransport(Graphics g)
    {
        Graphics2D g2 = (Graphics2D) g;
        int [] x ={_startPosX+5,_startPosX+80,_startPosX+120,_startPosX+80,_startPosX+5,_startPosX+5};
        int [] y ={_startPosY,_startPosY,_startPosY+20,_startPosY+40,_startPosY+40,_startPosY};
        if (_startPosX < 0 || _startPosY < 0 || _pictureHeight== null || _pictureWidth== null)
        {
            return;
        }

        g2.setColor(Warship.GetBodyColor());

        g2.fillPolygon(x,y,6);

        g2.setColor(Color.BLACK);

        g2.drawLine(_startPosX + 5, _startPosY, _startPosX + 80, _startPosY);
        g2.drawLine(_startPosX + 80, _startPosY, _startPosX + 120, _startPosY + 20);
        g2.drawLine(_startPosX + 120, _startPosY + 20, _startPosX + 80, _startPosY + 40);
        g2.drawLine(_startPosX + 80, _startPosY + 40, _startPosX + 5, _startPosY + 40);
        g2.drawLine(_startPosX + 5, _startPosY + 40, _startPosX + 5, _startPosY);

        g2.setColor(Color.BLACK);

        g2.fillRect(_startPosX, _startPosY + 5, 5, 10);
        g2.fillRect(_startPosX, _startPosY + 25, 5, 10);

        g2.setColor(Color.BLUE);

        g2.drawOval(_startPosX + 70, _startPosY + 15, 10, 10);
        g2.fillOval(_startPosX + 70, _startPosY + 15, 10, 10);

        Blocks.DrawBlock(g2, _startPosX,_startPosY);
    }

    public void ChangeBorders(int width,int height)
    {
        _pictureWidth = width;
        _pictureHeight = height;
        if (_pictureWidth<=_warshipWidth||_pictureHeight<=_warshipHeight)
        {
            _pictureWidth = null;
            _pictureHeight = null;
            return;
        }
        if (_startPosX + _warshipWidth > _pictureWidth)
        {
            _startPosX = _pictureWidth - _warshipWidth;
        }
        if (_startPosY + _warshipHeight > _pictureHeight)
        {
            _startPosY = _pictureHeight - _warshipHeight;
        }
    }
    public float[] GetCurrentPosition() {
        float[] dim = new float[4];
        dim[0] = _startPosX;
        dim[1] =_startPosY;
        dim[2] = _startPosX + _warshipWidth;
        dim[3] = _startPosY + _warshipHeight;
        return dim;
    }
}
