import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Optional;

public class FormMapWithSetWarships extends JFrame{

    private JComboBox СomboBoxSelectorMap;
    private JButton ButtonAddWarship;
    private JTextField TextBoxPosition;
    private JButton ButtonRemoveWarship;
    private JButton ButtonShowStorage;
    private JButton ButtonShowOnMap;
    private JButton ButtonDown;
    private JButton ButtonUp;
    private JButton ButtonLeft;
    private JButton ButtonRight;
    private JPanel PictureBox;
    private JPanel MainPanel;
    private JPanel GroupBoxTools;
    private JTextField TextFieldMap;
    private JButton CreateMapButton;
    private JButton DeleteMapButton;
    private JList ListBoxMaps;
    private JPanel MapPanel;
    private JButton ButtonShowDeleted;
    private Image bufferedImage;
    private MapsCollection _mapsCollection;
    private final HashMap<String,AbstractMap> _mapsDict=new HashMap<String,AbstractMap>(){
        {
            put("Первая карта",new SimpleMap());
            put("Вторая карта",new SecondMap());
        }
    };

    public FormMapWithSetWarships(){
        setTitle("Военный корабль");
        setContentPane(MainPanel);
        setResizable(false);
        setSize(1000,685);
        ShowWindow();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);
        _mapsCollection=new MapsCollection(getWidth(),getHeight());
        СomboBoxSelectorMap.removeAllItems();
        for(String elem:_mapsDict.keySet()){
            СomboBoxSelectorMap.addItem(elem);
        }
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        if (bufferedImage != null) {
            PictureBox.paintComponents(bufferedImage.getGraphics());
            PictureBox.getGraphics().drawImage(bufferedImage, 0, 0, null);
        }
    }

    private void ReloadMaps()
    {
        int index = ListBoxMaps.getSelectedIndex();
        ListBoxMaps.setListData(_mapsCollection.Keys().toArray(new String[0]));

        if (ListBoxMaps.getModel().getSize() > 0 && (index == -1 || index >= ListBoxMaps.getModel().getSize()))
        {
            ListBoxMaps.setSelectedIndex(0);
        }
        else if (ListBoxMaps.getModel().getSize() > 0 && index > -1 && index < ListBoxMaps.getModel().getSize())
        {
            ListBoxMaps.setSelectedIndex(index);
        }
        repaint();
    }

    private void ShowWindow(){

        ListBoxMaps.addListSelectionListener(e -> {
            if(ListBoxMaps.getSelectedIndex()==-1)
                return;
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
            repaint();
        });

        DeleteMapButton.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() == -1)
            {
                return;
            }
            if(JOptionPane.showConfirmDialog(this,"Удалить карту "+ListBoxMaps.getSelectedValue().toString()+"?",
                    "Удаление",JOptionPane.YES_NO_OPTION)==0)
            {
                _mapsCollection.DelMap(ListBoxMaps.getSelectedValue().toString());
                ReloadMaps();
            }
        });

        CreateMapButton.addActionListener(e -> {
            if (СomboBoxSelectorMap.getSelectedIndex() == -1 || TextFieldMap.getText()==null || TextFieldMap.getText().equals(""))
            {
                JOptionPane.showMessageDialog(this,"Не все данные заполнены","Ошибка",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (!_mapsDict.containsKey(СomboBoxSelectorMap.getSelectedItem()))
            {
                JOptionPane.showMessageDialog(this,"Нет такой карты","Ошибка",JOptionPane.ERROR_MESSAGE);
                return;
            }
            _mapsCollection.AddMap(TextFieldMap.getText(), _mapsDict.get(СomboBoxSelectorMap.getSelectedItem().toString()));
            ReloadMaps();
        });

        ButtonShowOnMap.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowOnMap();
            repaint();
        });

        ButtonShowStorage.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
            repaint();
        });

        ButtonAddWarship.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            FormCreater dialog=new FormCreater();
            dialog.setSize(1200,700);
            dialog.setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
            dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
            dialog.setVisible(true);
            if (dialog.getSelectedWarship()!=null) {
                DrawingObjectWarship warship = new DrawingObjectWarship(dialog.getSelectedWarship());

                if (_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).plus(warship)>=0) {
                    JOptionPane.showMessageDialog(this, "Объект добавлен", "Успех", JOptionPane.INFORMATION_MESSAGE);
                    bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                    repaint();
                } else {
                    JOptionPane.showMessageDialog(this, "Не удалось добавить объект", "Ошибка",JOptionPane.INFORMATION_MESSAGE);
                }
            }
        });

        ButtonRemoveWarship.addActionListener(e -> {
            String txt=TextBoxPosition.getText();
            if (txt==null||ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            int result = JOptionPane.showConfirmDialog(
                    this,
                    "Удалить объект?",
                    "Удаление",
                    JOptionPane.YES_NO_CANCEL_OPTION);
            if (result!=0)
                return;
            int pos = Integer.parseInt(txt);
            if(_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).minus(pos)!=null){
                JOptionPane.showMessageDialog(this, "Объект удален", "Успех", JOptionPane.INFORMATION_MESSAGE);
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                repaint();
            }
            else
            {
                JOptionPane.showMessageDialog(this,"Не удалось удалить объект","Ошибка",JOptionPane.INFORMATION_MESSAGE);
            }
        });

        ButtonShowDeleted.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex()==-1)
                return;
            DrawingObjectWarship warship=(DrawingObjectWarship)_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).GetWarshipInDeleted();
            if(warship!=null){
                FormWarship dialog=new FormWarship();
                dialog.SetDrawingWarship(warship.GetWarship());
                dialog.setSize(1200,700);
                dialog.setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
                dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
                dialog.setVisible(true);
            }
        });

        ButtonUp.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Up);
                repaint();
            }
        });

        ButtonDown.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Down);
                repaint();
            }
        });

        ButtonRight.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Right);
                repaint();
            }
        });

        ButtonLeft.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Left);
                repaint();
            }
        });
    }
}
