import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.LinkedList;

public class MapWithSetWarshipsGeneric<T extends IDrawingObject,U extends AbstractMap> {
    private final int _pictureWidth;
    private final int _pictureHeight;
    private final int _placeSizeWidth = 120;
    private final int _placeSizeHeight = 50;
    private final SetWarshipsGeneric<T> _setWarship;
    private final LinkedList<T> DeletedWarships;
    private final U _map;

    public MapWithSetWarshipsGeneric(int picWidth,int picHeight, U map)
    {
        int width = picWidth / _placeSizeWidth;
        int height = picHeight/_placeSizeHeight;
        _setWarship = new SetWarshipsGeneric<T>(width * height);
        _pictureWidth = picWidth;
        _pictureHeight = picHeight;
        _map = map;
        DeletedWarships=new LinkedList<>();
    }

    public int plus(T warship)
    {
        return _setWarship.Insert(warship);
    }

    public T minus(int position)
    {
        T warship=_setWarship.Remove(position);
        DeletedWarships.push(warship);
        return warship;
    }

    public Image ShowSet()
    {
        BufferedImage bmp = new BufferedImage(_pictureWidth, _pictureWidth,BufferedImage.TYPE_INT_ARGB);
        Graphics gr = bmp.getGraphics();
        DrawBackground(gr);
        DrawWarship(gr);
        return bmp;
    }

    public Image ShowOnMap()
    {
        Shaking();
        for (IDrawingObject warship : _setWarship)
        {
            return _map.CreateMap(_pictureWidth, _pictureHeight, warship);
        }
        return new BufferedImage(_pictureWidth, _pictureHeight,1);
    }

    public Image MoveObject(Direction direction)
    {
        if (_map != null)
        {
            return _map.MoveObject(direction);
        }
        return new BufferedImage(_pictureWidth, _pictureHeight,1);
    }

    public void Shaking()
    {
        int j = _setWarship.Count() - 1;
        for (int i = 0; i < _setWarship.Count(); i++)
        {
            if (_setWarship.Get(i) == null)
            {
                for (; j > i; j--)
                {
                    T warship = _setWarship.Get(j);
                    if (warship != null)
                    {
                        _setWarship.Insert(warship, i);
                        _setWarship.Remove(j);
                        break;
                    }
                }
                if (j <= i)
                {
                    return;
                }
            }
        }
    }

    private void DrawBackground(Graphics gr)
    {
        Graphics2D g=(Graphics2D)gr;

        Color brush=Color.BLACK;
        Stroke penWide = new BasicStroke(5);
        Stroke penThin = new BasicStroke(1);

        for (int i = 0; i < _pictureWidth / _placeSizeWidth; i++)
        {
            for (int j = 0; j < _pictureHeight / _placeSizeHeight + 1; ++j)
            {
                g.setColor(brush);
                g.setStroke(penWide);
                g.drawLine(i * _placeSizeWidth + 20, j * _placeSizeHeight+2, i * _placeSizeWidth + (int)(_placeSizeWidth*0.8), j * _placeSizeHeight+2);
                g.drawLine(i * _placeSizeWidth + 20, j * _placeSizeHeight + _placeSizeHeight/2+2, i * _placeSizeWidth + (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + _placeSizeHeight/2+2);
                g.drawLine(i * _placeSizeWidth + (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + 2, i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight + _placeSizeHeight/2);
                g.drawLine(i * _placeSizeWidth + _placeSizeWidth, j * _placeSizeHeight + _placeSizeHeight / 2, i * _placeSizeWidth+ (int)(_placeSizeWidth * 0.8), j * _placeSizeHeight + _placeSizeHeight);
                g.setStroke(penThin);
            }
        }
    }

    private void DrawWarship(Graphics gr)
    {
        int i=0;
        int width = _pictureWidth / _placeSizeWidth;
        int height = _pictureHeight / _placeSizeHeight;

        for (IDrawingObject warship : _setWarship)
        {
            if (warship==null)
                return;
            _setWarship.Get(i).SetObject(i % width * _placeSizeWidth, (height - 1 - i / width) * _placeSizeHeight, _pictureWidth, _pictureHeight);
            _setWarship.Get(i).DrawingObject(gr);
            i++;
        }
    }

    public T GetWarshipInList(int i){
        return _setWarship.Get(i);
    }

    public T GetWarshipInDeleted() {
        if(DeletedWarships.isEmpty())
            return null;
        return DeletedWarships.pop();
    }
}
